var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DialogProxyTools } from "../proxy/DialogProxyTools";
import { RecordSetVisitor } from "../proxy/RecordSetVisitor";
import { RecordVisitor } from "../proxy/RecordVisitor";
import { SessionVisitor } from "../proxy/SessionVisitor";
import { storage } from "../storage";
import { Log } from "../util/Log";
import { BriefcaseVisitor } from "./BriefcaseVisitor";
import { Briefcase_Briefcase_Details_RECORD } from "./samples/Briefcase_Briefcase_Details_RECORD";
import { SdaDialogDelegateStateVisitor } from "./SdaDialogDelegateStateVisitor";
export class SdaDialogDelegateTools {
    static constructAddToBriefcaseNullRedirection(tenantId, sessionId, referringDialogId) {
        const nullRedirectionId = DialogProxyTools.constructNullRedirectionId();
        return {
            "tenantId": tenantId,
            "referringObject": {
                "dialogMode": "LIST",
                "dialogAlias": "Workpackage_AddToBriefcase_FORM",
                "actionId": "alias_AddToBriefcase",
                "type": "hxgn.api.dialog.ReferringDialog",
                "dialogId": referringDialogId
            },
            "refreshNeeded": true,
            "sessionId": sessionId,
            "id": nullRedirectionId,
            "type": "hxgn.api.dialog.NullRedirection"
        };
    }
    static constructEnterOfflineModeNullRedirection(tenantId, sessionId) {
        return this.constructBriefcaseNullRedirection(tenantId, sessionId, this.ENTER_OFFLINE_MODE_MENU_ACTION_ID);
    }
    static constructExitOfflineModeNullRedirection(tenantId, sessionId) {
        return this.constructBriefcaseNullRedirection(tenantId, sessionId, this.EXIT_OFFLINE_MODE_MENU_ACTION_ID);
    }
    static constructOfflineLogoutResponse(sessionId) {
        return {
            "sessionId": sessionId,
            "type": this.SESSION_ID_MODEL_TYPE
        };
    }
    static constructRemoveFromBriefcaseNullRedirection(tenantId, sessionId, referringDialogId) {
        const nullRedirectionId = DialogProxyTools.constructNullRedirectionId();
        return {
            "tenantId": tenantId,
            "referringObject": {
                "dialogMode": "LIST",
                "dialogAlias": "Workpackage_RemoveFromBriefcase_FORM",
                "actionId": "alias_RemoveFromBriefcase",
                "type": "hxgn.api.dialog.ReferringDialog",
                "dialogId": referringDialogId
            },
            "refreshNeeded": true,
            "sessionId": sessionId,
            "id": nullRedirectionId,
            "type": "hxgn.api.dialog.NullRedirection"
        };
    }
    static getOfflineDocumentsDialogListId(workPackageId) {
        return this.OFFLINE_DOCUMENTS_DIALOG_LIST_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineDocumentsDialogPropertiesId(workPackageId) {
        return this.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineDocumentsDialogRootId(workPackageId) {
        return this.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineTagsDialogListId(workPackageId) {
        return this.OFFLINE_TAGS_DIALOG_LIST_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineTagsDialogPropertiesId(workPackageId) {
        return this.OFFLINE_TAGS_DIALOG_PROPERTIES_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineTagsDialogRootId(workPackageId) {
        return this.OFFLINE_TAGS_DIALOG_ROOT_ID.replace('${workPackageId}', workPackageId);
    }
    static isBriefcaseWorkbenchActionRequest(dialogRequest) {
        return dialogRequest.isPostWorkbenchActionPathWithActionId(SdaDialogDelegateTools.BRIEFCASE_WORKBENCH_ACTION_ID);
    }
    static isEnterOfflineModeMenuActionRequest(dialogRequest) {
        return dialogRequest.isPostMenuActionPathWithActionId(SdaDialogDelegateTools.ENTER_OFFLINE_MODE_MENU_ACTION_ID);
    }
    static isExitOfflineModeMenuActionRequest(dialogRequest) {
        return dialogRequest.isPostMenuActionPathWithActionId(SdaDialogDelegateTools.EXIT_OFFLINE_MODE_MENU_ACTION_ID);
    }
    static isOfflineBriefcaseCommentsRecordSetRequest(dialogRequest) {
        return dialogRequest.isPostRecordsPathWithDialogId(this.OFFLINE_BRIEFCASE_DIALOG_COMMENTS_ID);
    }
    static isOfflineBriefcaseDialogId(dialogId) {
        return dialogId === this.OFFLINE_BRIEFCASE_DIALOG_ROOT_ID;
    }
    static isOfflineBriefcaseDetailsDialogId(dialogId) {
        return dialogId === this.OFFLINE_BRIEFCASE_DIALOG_DETAILS_ID;
    }
    static isOfflineBriefcaseWorkPackagesDialogId(dialogId) {
        return dialogId === this.OFFLINE_BRIEFCASE_DIALOG_WORK_PACKAGES_ID;
    }
    static isOfflineBriefcaseWorkPackagesRequest(dialogRequest) {
        if (!dialogRequest.isPostRecordsPath()) {
            return false;
        }
        const pathFields = dialogRequest.deconstructPostRecordsPath();
        return SdaDialogDelegateTools.isOfflineBriefcaseWorkPackagesDialogId(pathFields.dialogId);
    }
    static isOfflineDocumentContentRequest(dialogRequest) {
        // TODO: This will process ANY content request, we need a specific navigation check
        return dialogRequest.isPostSessionContentPath();
    }
    static isOfflineDocumentOpenLatestFileMenuActionRequest(dialogRequest) {
        return dialogRequest.isPostMenuActionPathWithActionId(this.OPEN_LATEST_FILE_MENU_ACTION_ID);
    }
    static isOfflineDocumentsListRecordSetRequest(dialogRequest) {
        // TODO: Change "startsWith" to do a specific comparision -- then use DialogProxyTools::isPostRecordsAtDialogId()
        if (!dialogRequest.isPostRecordsPath()) {
            return false;
        }
        const pathFields = dialogRequest.deconstructPostRecordsPath();
        return pathFields.dialogId.startsWith(this.OFFLINE_DOCUMENTS_DIALOG_LIST_ID_PREFIX);
    }
    static isOfflineDocumentsPropertiesDialogId(dialogId) {
        return dialogId.startsWith(this.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID_PREFIX);
    }
    static isOfflineDocumentsRootDialogRequest(dialogRequest) {
        // TODO: Change "startsWith" to do a specific comparision -- then use DialogProxyTools::isGetDialogAtDialogId()
        if (!dialogRequest.isGetDialogPath()) {
            return false;
        }
        const pathFields = dialogRequest.deconstructGetDialogPath();
        return pathFields.dialogId.startsWith(this.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID_PREFIX);
    }
    static isOfflineShowTagsMenuActionRequest(dialogRequest) {
        return dialogRequest.isPostMenuActionPathWithActionId(this.SHOW_TAGS_MENU_ACTION_ID);
    }
    static isOfflineTagsListRecordSetRequest(dialogRequest) {
        // TODO: Change "startsWith" to do a specific comparision -- then use DialogProxyTools::isPostRecordsAtDialogId()
        if (!dialogRequest.isPostRecordsPath()) {
            return false;
        }
        const pathFields = dialogRequest.deconstructPostRecordsPath();
        return pathFields.dialogId.startsWith(this.OFFLINE_TAGS_DIALOG_LIST_ID_PREFIX);
    }
    static isOfflineTagsPropertiesDialogId(dialogId) {
        return dialogId.startsWith(this.OFFLINE_TAGS_DIALOG_PROPERTIES_ID_PREFIX);
    }
    static isOfflineTagsRootDialogRequest(dialogRequest) {
        // TODO: Change "startsWith" to do a specific comparision -- then use DialogProxyTools::isGetDialogAtDialogId()
        if (!dialogRequest.isGetDialogPath()) {
            return false;
        }
        const pathFields = dialogRequest.deconstructGetDialogPath();
        return pathFields.dialogId.startsWith(this.OFFLINE_TAGS_DIALOG_ROOT_ID_PREFIX);
    }
    static isOfflineWorkPackagesListRecordSetRequest(dialogRequest) {
        return dialogRequest.isPostRecordsPathWithDialogId(this.OFFLINE_WORK_PACKAGES_DIALOG_LIST_ID);
    }
    static isOfflineWorkPackagesOpenMenuActionRequest(dialogRequest) {
        return dialogRequest.isPostMenuActionPathWithDialogIdAndActionId(this.OFFLINE_WORK_PACKAGES_DIALOG_LIST_ID, this.OPEN_MENU_ACTION_ID);
    }
    static isOfflineWorkPackagesRootDialogRequest(dialogRequest) {
        return dialogRequest.isGetDialogPathWithDialogId(this.OFFLINE_WORK_PACKAGES_DIALOG_ROOT_ID);
    }
    static isWorkPackagesAddToBriefcaseMenuActionRequest(dialogRequest) {
        return dialogRequest.isPostMenuActionPathWithActionId(SdaDialogDelegateTools.ADD_TO_BRIEFCASE_MENU_ACTION_ID);
    }
    static isWorkPackagesListRecordSet(dialogRequest, jsonObject) {
        if (!dialogRequest.isPostRecordsPath()) {
            return false;
        }
        if (!jsonObject || !jsonObject.type || !jsonObject.dialogName) {
            return false;
        }
        return jsonObject.type === this.RECORD_SET_MODEL_TYPE &&
            jsonObject.dialogName === this.WORK_PACKAGES_LIST_DIALOG_NAME;
    }
    static isWorkPackagesRemoveFromBriefcaseMenuActionRequest(dialogRequest) {
        if (!dialogRequest.isPostMenuActionPath()) {
            return false;
        }
        const pathFields = dialogRequest.deconstructPostMenuActionPath();
        return pathFields.actionId === SdaDialogDelegateTools.REMOVE_FROM_BRIEFCASE_MENU_ACTION_ID;
    }
    static isWorkPackagesRootDialog(jsonObject) {
        if (!jsonObject || !jsonObject.type || !jsonObject.dialogName) {
            return false;
        }
        return jsonObject.type === this.EDITOR_DIALOG_MODEL_TYPE &&
            jsonObject.dialogName === this.WORK_PACKAGES_ROOT_DIALOG_NAME;
    }
    static insertBriefcaseMetaDataIntoWorkPackagesDialog(originalDialog) {
        const workPackagesQueryDialog = originalDialog.children[0];
        const propertyDefs = workPackagesQueryDialog.recordDef.propertyDefs;
        propertyDefs.push({
            "writeAllowed": false,
            "propertyName": "briefcase",
            "canCauseSideEffects": false,
            "upperCaseOnly": false,
            "propertyType": "boolean",
            "type": "hxgn.api.dialog.PropertyDef",
            "writeEnabled": false
        });
        const columns = workPackagesQueryDialog.view.columns;
        columns.push({
            "propertyName": "briefcase",
            "heading": "Briefcase",
            "type": "hxgn.api.dialog.Column"
        });
        // Return original dialog WITH patches
        return originalDialog;
    }
    static readDialogDelegateStateVisitor(tenantId, userId) {
        const key = this.createStorageKey(tenantId, userId, this.DIALOG_DELEGATE_STATE_KEY);
        return storage.getJson(key).then(jsonObject => {
            if (!jsonObject) {
                const briefcase = Briefcase_Briefcase_Details_RECORD.copyOfResponse();
                BriefcaseVisitor.visitAndSetOnline(briefcase, true);
                jsonObject = {
                    briefcase: Briefcase_Briefcase_Details_RECORD.copyOfResponse(),
                    selectedWorkPackageIds: [],
                    userId: null,
                    workPackages: RecordSetVisitor.emptyRecordSetVisitor().enclosedJsonObject(),
                    mobileComments: RecordSetVisitor.emptyRecordSetVisitor().enclosedJsonObject()
                };
                Log.info('SdaDialogDelegateTools::readDelegateState -- returning defaults: ' + JSON.stringify(jsonObject));
            }
            return new SdaDialogDelegateStateVisitor(jsonObject);
        });
    }
    static readOfflineDocumentsPropertiesRecord(tenantId, userId, dialogId) {
        const key = `${userId}.${tenantId}.ppm.sda.workPackages.documents.dialog.${dialogId}.record`;
        return storage.getJson(key).then(jsonObject => new RecordVisitor(jsonObject));
    }
    static readOfflineSession(tenantId, userId) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_SESSION_KEY);
        return storage.getJson(key).then(jsonObject => new SessionVisitor(jsonObject));
    }
    static readOfflineTagsPropertiesRecord(tenantId, userId, dialogId) {
        const key = `${userId}.${tenantId}.ppm.sda.workPackages.tags.dialog.${dialogId}.record`;
        return storage.getJson(key).then(jsonObject => new RecordVisitor(jsonObject));
    }
    static showAllStorageKeys() {
        return __awaiter(this, void 0, void 0, function* () {
            const thisMethod = 'SdaDialogDelegateTools::showAllStorageKeys';
            Log.info(`${thisMethod} -- ************** BEGIN SHOW ALL STORAGE KEYS **************`);
            const allKeys = yield storage.getAllKeys();
            for (const k of allKeys) {
                const v = yield storage.getItem(k);
                Log.info(`${thisMethod} -- ${k}`);
            }
            Log.info(`${thisMethod} -- ************** END SHOW ALL STORAGE KEYS **************`);
        });
    }
    static showAllStorageKeysAndValues() {
        return __awaiter(this, void 0, void 0, function* () {
            const thisMethod = 'SdaDialogDelegateTools::showAllStorageKeysAndValues';
            Log.info(`${thisMethod} -- ************** BEGIN SHOW ALL STORAGE KEYS AND VALUES **************`);
            const allKeys = yield storage.getAllKeys();
            for (const k of allKeys) {
                const v = yield storage.getItem(k);
                Log.info(`${thisMethod} -- ${k}: ${v}`);
            }
            Log.info(`${thisMethod} -- ************** END SHOW ALL STORAGE KEYS AND VALUES **************`);
        });
    }
    static writeDialogDelegateState(tenantId, stateVisitor) {
        const userId = stateVisitor.visitUserId();
        const key = this.createStorageKey(tenantId, userId, this.DIALOG_DELEGATE_STATE_KEY);
        return storage.setJson(key, stateVisitor.enclosedJsonObject());
    }
    static writeOfflineSession(tenantId, userId, offlineSessionVisitor) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_SESSION_KEY);
        return storage.setJson(key, offlineSessionVisitor.enclosedJsonObject());
    }
    static constructBriefcaseNullRedirection(tenantId, sessionId, actionId) {
        const nullRedirectionId = DialogProxyTools.constructNullRedirectionId();
        return {
            "tenantId": tenantId,
            "referringObject": {
                "dialogMode": "READ",
                "dialogAlias": "Briefcase_Briefcase_Details",
                "actionId": actionId,
                "type": "hxgn.api.dialog.ReferringDialog",
                "dialogId": this.OFFLINE_BRIEFCASE_DIALOG_DETAILS_ID
            },
            "refreshNeeded": true,
            "sessionId": sessionId,
            "id": nullRedirectionId,
            "type": "hxgn.api.dialog.NullRedirection"
        };
    }
    static createStorageKey(tenantId, userId, keyTemplate) {
        const key = keyTemplate.replace('${tenantId}', tenantId);
        return key.replace('${userId}', userId);
    }
}
// Dialog Ids
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_COMMENTS_ID = 'offline_briefcase_comments';
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_DETAILS_ID = 'offline_briefcase_details';
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_ROOT_ID = 'offline_briefcase';
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_WORK_PACKAGES_ID = 'offline_briefcase_workPackages';
SdaDialogDelegateTools.OFFLINE_WORK_PACKAGES_DIALOG_LIST_ID = 'offline_workPackages_list';
SdaDialogDelegateTools.OFFLINE_WORK_PACKAGES_DIALOG_ROOT_ID = 'offline_workPackages';
// Dialog Names
SdaDialogDelegateTools.BRIEFCASE_DETAILS_DIALOG_NAME = 'Briefcase_Briefcase_Details';
SdaDialogDelegateTools.BRIEFCASE_MOBILE_COMMENTS_DIALOG_NAME = 'Briefcase_Briefcase_MobileComments';
SdaDialogDelegateTools.BRIEFCASE_ROOT_DIALOG_NAME = 'Briefcase_Briefcase_FORM';
SdaDialogDelegateTools.BRIEFCASE_WORK_PACKAGES_DIALOG_NAME = 'Briefcase_Briefcase_Workpackages';
SdaDialogDelegateTools.DOCUMENTS_ROOT_DIALOG_NAME = 'Workpackage_Documents_FORM';
SdaDialogDelegateTools.DOCUMENTS_PROPERTIES_DIALOG_NAME = 'Workpackage_Documents_Properties';
SdaDialogDelegateTools.DOCUMENTS_LIST_DIALOG_NAME = 'Workpackage_Documents_Documents';
SdaDialogDelegateTools.TAGS_ROOT_DIALOG_NAME = 'Workpackage_Tags_FORM';
SdaDialogDelegateTools.TAGS_PROPERTIES_DIALOG_NAME = 'Workpackage_Tags_Properties';
SdaDialogDelegateTools.TAGS_LIST_DIALOG_NAME = 'Workpackage_Tags_Tags';
SdaDialogDelegateTools.WORK_PACKAGES_LIST_DIALOG_NAME = 'Workpackage_General';
SdaDialogDelegateTools.WORK_PACKAGES_ROOT_DIALOG_NAME = 'Workpackage_General_FORM';
// Dynamic Dialog Ids -- PRIVATE
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_LIST_ID_PREFIX = 'offline_documents_list_';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_LIST_ID = `${SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_LIST_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID_PREFIX = 'offline_documents_properties_';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID = `${SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID_PREFIX = 'offline_documents_';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID = `${SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_ID_PREFIX = 'offline_tags_list_';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_ID = `${SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_ID_PREFIX = 'offline_tags_properties_';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_ID = `${SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_ID_PREFIX = 'offline_tags_';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_ID = `${SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_ID_PREFIX}\${workPackageId}`;
// Action Ids
SdaDialogDelegateTools.ADD_TO_BRIEFCASE_MENU_ACTION_ID = 'alias_AddToBriefcase';
SdaDialogDelegateTools.BRIEFCASE_WORKBENCH_ACTION_ID = 'Briefcase';
SdaDialogDelegateTools.ENTER_OFFLINE_MODE_MENU_ACTION_ID = 'alias_EnterOfflineMode';
SdaDialogDelegateTools.EXIT_OFFLINE_MODE_MENU_ACTION_ID = 'alias_ExitOfflineMode';
SdaDialogDelegateTools.OPEN_MENU_ACTION_ID = 'alias_Open';
SdaDialogDelegateTools.OPEN_LATEST_FILE_MENU_ACTION_ID = 'alias_OpenLatestFile';
SdaDialogDelegateTools.REMOVE_FROM_BRIEFCASE_MENU_ACTION_ID = 'alias_RemoveFromBriefcase';
SdaDialogDelegateTools.SHOW_TAGS_MENU_ACTION_ID = 'alias_ShowTags';
// Model Types
SdaDialogDelegateTools.EDITOR_DIALOG_MODEL_TYPE = 'hxgn.api.dialog.EditorDialog';
SdaDialogDelegateTools.RECORD_SET_MODEL_TYPE = 'hxgn.api.dialog.RecordSet';
SdaDialogDelegateTools.SESSION_ID_MODEL_TYPE = 'hxgn.api.dialog.SessionId';
// Property Names
SdaDialogDelegateTools.ONLINE_PROPERTY_NAME = 'online';
// Storage Keys
SdaDialogDelegateTools.DIALOG_DELEGATE_STATE_KEY = '${userId}.${tenantId}.SdaDialogDelegate.state';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_REDIRECTION_KEY = '${userId}.${tenantId}.ppm.sda.workPackages.documents.redirection.${workPackageId}';
SdaDialogDelegateTools.OFFLINE_SESSION_KEY = '${userId}.${tenantId}.OfflineSession';
SdaDialogDelegateTools.OFFLINE_TAGS_REDIRECTION_KEY = '${userId}.${tenantId}.ppm.sda.workPackages.tags.redirection.${workPackageId}';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_KEY = '${userId}.${tenantId}.ppm.sda.workPackages.tags.dialog.offline_tags_${workPackageId}';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_RECORD_KEY = '${userId}.${tenantId}.ppm.sda.workPackages.tags.dialog.offline_tags_properties_${workPackageId}.record';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_RECORD_SET_KEY = '${userId}.${tenantId}.ppm.sda.workPackages.tags.dialog.offline_tags_list_${workPackageId}.recordSet';
